require_relative '../../../spec_helper'

describe "Process::GID.eid" do
  it "needs to be reviewed for spec completeness"
end

describe "Process::GID.eid=" do
  it "needs to be reviewed for spec completeness"
end
